MIN_LENGTH = 1
MAX_LENGTH = 400

IDENTIFIER_REGEX_CONDITION = r"^[A-Za-z0-9\.\-:_,=\/@]+$"
MSMQ_NAME_REGEX_CONDITION = r"^[A-Za-z0-9_]+$"


ORG = "ORG"
APPLICATION = "APP"
PROJECT = "PRJ"
TYPE = "TYPE"
NAMESPACE = "NS"



from rest_framework.exceptions import ValidationError
from rest_framework.exceptions import APIException
from rest_framework.views import Response, exception_handler
from rest_framework import status
from django.core.exceptions import ValidationError as CoreValidationError
from django.db import IntegrityError
from django.conf import settings
import json
import traceback
import requests

from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.core.common.logger import Logger, LogLevel

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


class KubefacetsAPIException(APIException):
    default_detail = "Unexpected Error"
    status_code = 500
    default_code = "unexpected_error"
    detailed_msg = ""

    def __init__(self, detail, defaultCode, statusCode, detailedMsg=""):
        self.detail = detail
        self.status_code = statusCode
        self.default_code = defaultCode
        self.detailed_msg = detailedMsg
        logger.error(self.detailed_msg)


def KubefacetsAPIExceptionHandler(ex, context):
    response = exception_handler(ex, context)
    request = context["request"]
    errorData = {
        "status": "ERROR",
        "path": request.path,
        "method": request.method,
        "content_type": request.content_type,
        "query_params": request.query_params,
        "body": request.data,
        "cookies": request.COOKIES,
        "error": ex.__str__(),
    }
    errorJson = json.dumps(errorData, indent=4)
    print(errorJson)

    # print error to console
    if settings.DEBUG:
        print("[[ STACKTRACE ]]")
        traceback.print_exc()

    if isinstance(ex, IntegrityError) and not response:
        response = Response(
            {"detail": ex.__str__(), "code": "bad_request"},
            status=status.HTTP_400_BAD_REQUEST,
        )

    if isinstance(ex, TypeError) and not response:
        response = Response(
            {"detail": ex.__str__(), "code": "system_error"},
            status=status.HTTP_500_INTERNAL_SERVER_ERROR,
        )

    if isinstance(ex, ValidationError):
        response = Response(
            {"detail": ex.args[0], "code": "bad_request"},
            status=status.HTTP_400_BAD_REQUEST,
        )

    if isinstance(ex, CoreValidationError):
        response = Response(
            {"detail": ex.args[0], "code": "bad_request"},
            status=status.HTTP_400_BAD_REQUEST,
        )

    if isinstance(ex, KubefacetsAPIException):
        response = Response(
            {"detail": ex.detail, "code": ex.default_code}, status=ex.status_code
        )
    return response


def handle_gateway_exception(f):
    def wrapper(*args, **kwargs):
        obj = args[0]
        try:
            resp = f(*args, **kwargs)
            return resp
        except KubefacetsAPIException as e:
            jsonData = {
                "status": False,
                "data": {},
                "error": {
                    "detail": e.detail,
                    "status_code": e.status_code,
                    "default_code": e.default_code,
                    "type": "error",
                    "detailed_msg": e.detailed_msg,
                },
            }
            resp = requests.Response()
            resp.status_code = status.HTTP_504_GATEWAY_TIMEOUT
            resp._content = json.dumps(jsonData).encode("utf-8")
            resp.headers["Content-Type"] = "application/json"
            obj.setResponse(resp)
            obj.getLogger().logWebRequestError(
                obj.getDjangoRequest().getRequest(),
                jsonData,
                DictUtils.get_by_path(jsonData, "error.type"),
            )
            return resp.json()

    return wrapper



from drf_spectacular.utils import OpenApiExample


class OutpostV1Doc:
    @staticmethod
    def modelviewset_list_path_examples():
        return []

    @staticmethod
    def modelviewset_list_examples():
        return []

    @staticmethod
    def modelviewset_get_path_examples():
        return []

    @staticmethod
    def modelviewset_get_examples():
        return []

    @staticmethod
    def modelviewset_create_examples():
        return [
            OpenApiExample(
                "Outpost - Create",
                value={
                    "msg_queue_info": {
                        "exchange_name": "remind_exchange",
                        "queue_name": "remind_queue",
                        "routing_key": "",
                    },
                    "msg": {
                        "action": "REORDER_CERT",
                        "type": "ONETIME",
                        "data": {
                            "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA",
                            "remind_by": "05/31/2023, 08:01:19 AM",
                            "to_msg_queue": {
                                "exchange_name": "certs_exchange",
                                "queue_name": "certs_queue",
                                "routing_key": "certs",
                            },
                            "body": {
                                "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA"
                            },
                        },
                    },
                },
                request_only=True,
                response_only=False,
            ),
        ]

    @staticmethod
    def modelviewset_delete_path_examples():
        return []

    @staticmethod
    def modelviewset_patch_path_examples():
        return []

    @staticmethod
    def modelviewset_patch_examples():
        return []



from drf_spectacular.utils import extend_schema_view

from kfsd.apps.models.tables.outpost import Outpost
from kfsd.apps.endpoints.serializers.common.outpost import OutpostViewModelSerializer
from kfsd.apps.endpoints.views.docs.outpost import OutpostDoc
from kfsd.apps.endpoints.views.common.custom_model import CustomModelViewSet


@extend_schema_view(**OutpostDoc.modelviewset())
class OutpostModelViewSet(CustomModelViewSet):
    queryset = Outpost.objects.all()
    serializer_class = OutpostViewModelSerializer



from kfsd.apps.core.utils.http.django.cookie import Cookie


class DjangoRequest():
    def __init__(self, request=None):
        self.__request = request
        self.__djangoCookies = Cookie(request)

    def getRequest(self):
        return self.__request

    def getDjangoReqCookies(self):
        return self.__djangoCookies

    def parseInputData(self, serializer, raiseExceptions=True):
        inputSerializer = serializer(data=self.__request.data)
        inputSerializer.is_valid(raise_exception=raiseExceptions)
        return inputSerializer.data



from kfsd.apps.core.tests.base_api import BaseAPITestCases
from django.urls import reverse
from unittest.mock import patch


class APIDocsViewTests(BaseAPITestCases):
    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    @patch("kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getLocalKubefacetsSettingsConfig")
    @patch("kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.deriveRemoteConfig")
    def test_get(self, remoteConfigMocked, rawSettingsMocked, tokenUserInfoMocked):
        rawConfig = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/common/kubefacets/remote_settings.json')
        remoteConfig = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/common/kubefacets/remote_config_resp.json')
        rawSettingsMocked.return_value = rawConfig
        remoteConfigMocked.return_value = remoteConfig
        staffUserInfoResp = {"status": True, "data": {"user": {"identifier": "123", "is_staff": True, "is_active": True, "is_email_verified": True}}}
        tokenUserInfoMocked.return_value = staffUserInfoResp
        url = reverse("api_doc")
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)



from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver
import json

from kfsd.apps.models.base import BaseModel
from kfsd.apps.core.utils.system import System
from kfsd.apps.models.constants import MAX_LENGTH
from kfsd.apps.core.msmq.rabbitmq.base import RabbitMQ


class Outpost(BaseModel):
    STATUS_CHOICES = (
        ("P", "PENDING"),
        ("I", "IN-PROGRESS"),
        ("C", "COMPLETED"),
    )

    msg_queue_info = models.JSONField(default=dict)
    msg = models.JSONField(default=dict)
    status = models.CharField(max_length=1, choices=STATUS_CHOICES, default="P")
    attempts = models.IntegerField(default=0)
    debug_info = models.CharField(max_length=MAX_LENGTH, default="")

    def save(self, *args, **kwargs):
        self.identifier = System.uuid(32)
        return super().save(*args, **kwargs)

    class Meta:
        app_label = "models"
        verbose_name = "Outpost"
        verbose_name_plural = "Outpost"


@receiver(post_save, sender=Outpost)
def send_msg(sender, instance, created, **kwargs):
    if created:
        rabbitMQ = RabbitMQ()
        rabbitMQ.publish_msg_and_close_connection(
            instance.msg_queue_info["exchange_name"],
            instance.msg_queue_info["queue_name"],
            instance.msg_queue_info["routing_key"],
            json.dumps(instance.msg),
        )



from django.core.management.base import BaseCommand
from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.core.msmq.rabbitmq.base import RabbitMQ


class Command(BaseCommand):
    help = "Listens to a RabbitMQ topic"

    def getConnectionArguments(self):
        connectionConfig = DictUtils.get_by_path(
            self.__config, "services.rabbitmq.connect"
        )
        authCredentials = connectionConfig.pop("credentials")
        connectionConfig["credentials"] = RabbitMQ.constructCredentials(
            DictUtils.get(authCredentials, "username"),
            DictUtils.get(authCredentials, "pwd"),
        )
        return connectionConfig

    def handle(self, *args, **options):
        msmqHandler = RabbitMQ()
        self.__config = msmqHandler.getConfig()
        exchangeName = "test_exchange"
        queueName = "test_queue"
        routingKey = "test_routing"

        msmqHandler.publish_msg(
            exchangeName, queueName, routingKey, "hi this is my first msg"
        )

        def callback(ch, method, properties, body):
            print("Msg Body: {}".format(body))

        msmqHandler.consume_msgs(callback, exchangeName, queueName, routingKey)



from kfsd.apps.core.utils.http.base import HTTP
from kfsd.apps.core.utils.http.django.request import DjangoRequest
from kfsd.apps.core.utils.http.headers.base import Headers
from kfsd.apps.core.utils.http.headers.contenttype import ContentType
from kfsd.apps.core.common.kubefacets_config import KubefacetsConfig
from kfsd.apps.core.utils.http.django.config import DjangoConfig


class APIGateway(Headers, HTTP):
    def __init__(self, request=None):
        self.__request = DjangoRequest(request)
        self.__config = DjangoConfig(KubefacetsConfig().getConfig())
        Headers.__init__(self)
        HTTP.__init__(self)

    def getServicesAPIKey(self):
        return self.getDjangoConfig().findConfigs(["services.api_key"])[0]

    def getDjangoRequest(self):
        return self.__request

    def getDjangoConfig(self):
        return self.__config

    def setHttpHeaders(self):
        self.setAPIKey(self.getServicesAPIKey())
        self.setContentType(ContentType.APPLICATION_JSON)

    def httpPost(self, postUrl, payload, expStatus):
        self.setHttpHeaders()
        resp = self.post(postUrl, expStatus, json=payload, headers=self.getReqHeaders())
        return resp.json()

    def httpGet(self, getUrl, expStatus):
        self.setHttpHeaders()
        resp = self.get(getUrl, expStatus, headers=self.getReqHeaders())
        return resp.json()

    def httpDel(self, getUrl, expStatus):
        self.setHttpHeaders()
        self.delete(getUrl, expStatus, headers=self.getReqHeaders())

    def constructUrl(self, configPaths):
        uris = self.getDjangoConfig().findConfigs(configPaths)
        return self.formatUrl(uris)
